/* Example program calling g13bec followed by g13bjc for cases for simple seasonal ARIMA models
   (i.e. no input series)
*/
#include <nag.h>
#include <stdio.h>
#include <nag_string.h>
#include <nag_stdlib.h>
#include <nagg13.h>

int main(void)
{
  Nag_ArimaOrder arimav;
  Nag_TransfOrder transfv;
  double df, objf, rss;
  double rmsxy[1], *para=0, *sd=0, *xxy=0, *fva=0, *fsd=0, *parx=0;
  Integer exit_status=0, i, npara, nseries, nxxy, tdxxy, nev, tdmrx, ldparx, tdparx, nfv;
  Integer *mrx=0;

  Vprintf("nag_tsa_multi_inp_model_estim (g13bec) Example Program Results\n");

  /* There is only a single series (y, called the output series in the g13bec documentation) */
  nseries = 1;

  /* As we only have one series, and are dynamically allocating memory, we can set the stride to nseries (i.e. 1) */
  tdxxy = 1;

  /* Allocate memory to transfer function model orders (even though it is not being used) 
     this parameter does not need to be initialised */
  nag_tsa_transf_orders(nseries, &transfv, NAGERR_DEFAULT);

  /* Skip heading in data file */
  Vscanf(" %*[^\n]");

  /* Read in the problem size and the number of values to forecast */
  Vscanf("%"NAG_IFMT" %"NAG_IFMT" %*[^\n]", &nxxy,&nfv);
  if (nxxy < 1) {
    Vprintf( "Problem has no data, nxxy = %"NAG_IFMT"\n", nxxy);
    exit_status = 1;
    goto END;
  }

  /* Make sure that the number of forecast values is not negative */
  if (nfv < 0) nfv = 0;

  /* Read the orders vector of the ARIMA model for the output noise component into structure arimav */
  Vscanf("%"NAG_IFMT" %"NAG_IFMT" %"NAG_IFMT" %"NAG_IFMT" %"NAG_IFMT" %"NAG_IFMT" %"NAG_IFMT" %*[^\n]", 
         &arimav.p, &arimav.d, &arimav.q, &arimav.bigp, &arimav.bigd, &arimav.bigq, &arimav.s);

  npara = arimav.p + arimav.q + arimav.bigp + arimav.bigq + nseries;
  if (npara < 1) {
    Vprintf( "No parameters are being estimated, npara = %"NAG_IFMT"\n", npara);
    exit_status = 1;
    goto END;
  }
    
  /* Allocate memory for the data, and output parameters. As we are forecasting nfv values, xxy must be large
   enough to hold the original nxxy data points + the forecasted values */
  if ( !( para = NAG_ALLOC(npara, double)) ||
       !( sd = NAG_ALLOC(npara, double)) ||
       !( xxy = NAG_ALLOC((nxxy+nfv)*tdxxy, double)) ) {
      Vprintf("Allocation failure\n");
      exit_status = -1;
      goto END;
  }

  /* Read in some initial values for the parameters */
  for (i=0; i<npara; ++i)
    Vscanf("%lf", &para[i]);
  Vscanf(" %*[^\n]");

  /* Read in the observed series (skip comments on the first observation) */
  Vscanf("%lf %*[^\n]", &xxy[0]);
  for (i=1; i<nxxy; ++i)
    Vscanf("%lf", &xxy[i]);

  Vprintf("\n\n====================================\n");
  Vprintf("Fitting the ARIMA model (%"NAG_IFMT" %"NAG_IFMT" %"NAG_IFMT")(%"NAG_IFMT" %"NAG_IFMT" %"NAG_IFMT") %"NAG_IFMT" using g13bec\n\n",
          arimav.p,arimav.d,arimav.q,arimav.bigp,arimav.bigd,arimav.bigq,arimav.s);

  /* Estimate the parmameters for the time series model */
  nag_tsa_multi_inp_model_estim(&arimav, nseries, &transfv, para, npara,nxxy, xxy, tdxxy, sd, &rss, &objf, &df,
                                G13_DEFAULT, NAGERR_DEFAULT);

  /* If required forecast some additional values */
  if (nfv > 0) {
    Vprintf("\n\n====================================\n");
    Vprintf("Forecasting %"NAG_IFMT" values using g13bjc\n\n", nfv);

    /* Allocate some memory to hold the forecast information */
    if ( !( fva = NAG_ALLOC(nfv, double)) ||
         !( fsd = NAG_ALLOC(nfv, double))) {
      Vprintf("Allocation failure\n");
      exit_status = -1;
      goto END;
    }

    /* nev is the number of observed values for the output (y) series */
    nev = nxxy;

    /* As we do not have any input (x) series, then rmsxy need not be initialised, mrx and parx can be set
       to null and tdmrx, ldparx and tdparx set to 1 */
    mrx = 0;
    parx = 0;
    tdmrx = ldparx = tdparx = 1;
    nag_tsa_multi_inp_model_forecast(&arimav, nseries, &transfv, para,
                                     npara, nev, nfv, xxy, tdxxy, rmsxy,
                                     mrx, tdmrx, parx, ldparx, tdparx,
                                     fva, fsd, G13_DEFAULT, NAGERR_DEFAULT);
        
    Vprintf("The forecast values and their standard errors are\n");
    Vprintf("\n   i       fva       fsd\n\n");
    for (i=0; i<nfv; ++i)
      Vprintf("%4"NAG_IFMT" %10.3f %10.4f\n", i+1, fva[i], fsd[i]);    

    Vprintf("\nThe residual mean square = %10.5e\n", rmsxy[0]);
  }

 END:
  nag_tsa_trans_free(&transfv);
  if (para) NAG_FREE(para);
  if (sd) NAG_FREE(sd);
  if (xxy) NAG_FREE(xxy);
  if (fva) NAG_FREE(fva);
  if (fsd) NAG_FREE(fsd);
  return exit_status;
}
